from imports import *


class index:
    ####################################################################################// Load
    def __init__(self, app="", cwd="", args=[]):
        self.app, self.cwd, self.args = app, cwd, args

        self.sources = os.path.join(self.app, ".system/sources")
        self.on = False
        pass

    def __exit__(self):
        if Localhost.check():
            if self.on:
                cli.done(datetime.now().strftime("%Y-%m-%d %H:%M:%S") + "\n")
            self.stop()
        pass

    ####################################################################################// Main
    def start(self, domain=""):  # (domain) - Start the project with virtual domain
        domain = "localhost" if not domain.strip() else domain.strip()
        Localhost.start("xamphp", self.sources, self.cwd, {"domain": domain})

        self.on = True
        cli.done(datetime.now().strftime("%Y-%m-%d %H:%M:%S"))

        N = 0
        while True:
            N = 1 if N >= 3 else N + 1
            cli.done(("." * N) + "      ", True)
            time.sleep(2)
        pass

    def stop(self):  # Stop the project if it didn't
        Localhost.stop("xamphp", self.sources, self.cwd)
        pass

    ####################################################################################// Helpers



from imports import *


class Localhost:
    ####################################################################################// Load
    def __init__(self, cliname="", sources="", current=""):
        self.cliname = cliname
        self.sources = sources
        self.current = current
        pass

    ####################################################################################// Main
    def start(cliname="", sources="", current="", info={}):
        if (
            not cliname
            or not os.path.exists(sources)
            or not os.path.exists(current)
            or not info
        ):
            return False

        obj = Localhost(cliname, sources, current)
        return obj.__startLocalhost(info)

    def stop(cliname="", sources="", current=""):
        if not cliname or not os.path.exists(sources) or not os.path.exists(current):
            return False

        obj = Localhost(cliname, sources, current)
        obj.__stopLocalhost()
        pass

    def check():
        apache = r"C:/xampp/apache/logs/httpd.pid"
        mysql = r"C:/xampp/mysql/data/mysql.pid"

        if not os.path.exists(apache) and not os.path.exists(mysql):
            return False

        return True

    ####################################################################################// Helpers
    def __startLocalhost(self, config={}):
        server = r"C:/xampp/xampp_start.exe"

        if not os.path.exists(server):
            cli.error("XAMPP not found: 'C:/xampp'")
            return False

        if "domain" not in config or not config["domain"]:
            cli.error("Invalid domain name")
            return False

        domain = config["domain"]
        if not self.__setVirtualHost(domain):
            return False

        if not self.__setHost(domain):
            return False

        cli.done("Please wait ...")
        if not self.__execute(server, "XAMPP started"):
            return False

        print()
        cli.hint(f"Apache: http://{domain}")
        cli.hint(f"MySQL: http://{domain}/phpmyadmin")
        print()

        webbrowser.open(f"http://{domain}")

        return True

    def __stopLocalhost(self):
        self.__resetVirtualHost()
        self.__resetHost()

        server = r"C:/xampp/xampp_stop.exe"
        if not os.path.exists(server):
            cli.error("Not found: 'C:/xampp'")
            return False

        if not Localhost.check():
            cli.done("XAMPP is already stopped")
            return True

        self.__execute(server, "XAMPP stopped", True)
        pass

    def __setVirtualHost(self, domain=""):
        if not domain:
            cli.error("Invalid VirtualHost domain")
            return False

        self.__resetVirtualHost()

        file = "C:/xampp/apache/conf/extra/httpd-vhosts.conf"
        if not os.path.exists(file):
            cli.error("Config not found: vhosts.conf")
            return False

        tmpl = os.path.join(self.sources, "vhosts.conf")
        if not os.path.exists(tmpl):
            cli.error("Invalid template: vhosts.conf")
            return False

        template = cli.read(tmpl)
        replaced = cli.template(
            template,
            {"cliname": self.cliname, "current": self.current, "domain": domain},
        )
        if not template or not replaced:
            cli.error("Invalid template content: vhosts.conf")
            return False

        content = cli.read(file) + "\n\n" + replaced
        if not cli.write(file, content):
            cli.error("Config failed: vhosts.conf")
            return False

        cli.done("VirtualHost configured")
        return True

    def __resetVirtualHost(self):
        file = "C:/xampp/apache/conf/extra/httpd-vhosts.conf"
        if not os.path.exists(file):
            cli.error("Not found: vhosts.conf")
            return False

        content = cli.read(file)
        pattern = rf"# {self.cliname}-vhost(.*?)</VirtualHost>"
        matches = re.findall(pattern, content, re.DOTALL)

        if not matches:
            return True
        for match in matches:
            content = content.replace(
                f"\n\n# {self.cliname}-vhost{match}</VirtualHost>", ""
            )
            content = content.replace(
                f"\n# {self.cliname}-vhost{match}</VirtualHost>", ""
            )

        if not cli.write(file, content):
            cli.error("Failed: vhosts.conf")
            return False

        cli.done("VirtualHost removed")
        return True

    def __setHost(self, domain=""):
        if not domain:
            cli.error("Invalid Host domain")
            return False

        self.__resetHost()

        file = "C:/Windows/System32/drivers/etc/hosts"
        if not os.path.exists(file):
            cli.error("Config not found: hosts")
            return False

        tmpl = os.path.join(self.sources, "hosts")
        if not os.path.exists(tmpl):
            cli.error("Invalid template: hosts")
            return False

        template = cli.read(tmpl)
        replaced = cli.template(template, {"cliname": self.cliname, "domain": domain})
        if not template or not replaced:
            cli.error("Invalid template content: hosts")
            return False

        content = cli.read(file) + "\n\n" + replaced
        if not cli.write(file, content):
            cli.error("Config failed: hosts")
            return False

        cli.done("Host configured")
        return True

    def __resetHost(self):
        file = "C:/Windows/System32/drivers/etc/hosts"
        if not os.path.exists(file):
            cli.error("Not found: hosts")
            return False

        content = cli.read(file)
        pattern = rf"# {self.cliname}-hosts(.*?)# {self.cliname}-host"
        matches = re.findall(pattern, content, re.DOTALL)

        if not matches:
            return True
        for match in matches:
            content = content.replace(
                f"\n\n# {self.cliname}-hosts{match}# {self.cliname}-host", ""
            )
            content = content.replace(
                f"\n# {self.cliname}-hosts{match}# {self.cliname}-host", ""
            )

        if not cli.write(file, content):
            cli.error("Failed: hosts")
            return False

        cli.done("Host removed")
        return True

    def __execute(self, line="", message="", background=False):
        if not line:
            cli.error("Invalid CMD line")
            return False

        try:
            if background:
                subprocess.Popen(line, shell=True)
            else:
                subprocess.run(line, check=True)
            cli.done(message)
            return True
        except subprocess.CalledProcessError:
            cli.error(f"CMD Failed: {message}")
            return False

        return False



from clight.system.importer import cli  # DON'T REMOVE THIS LINE

import os
import re
import sys
import time
import subprocess
import webbrowser
from datetime import datetime
from modules.localhost import Localhost



from setuptools import setup, find_packages

setup(
    name="xamphp",
    version="1.0.2",
    packages=find_packages(),
    install_requires=[
        "clight",
        
    ],
    entry_points={
        "console_scripts": [
            "xamphp=xamphp.main:main",  # Entry point of the app
        ],
    },
    package_data={
        "xamphp": [
            "main.py",
            "__init__.py",
            ".system/imports.py",
            ".system/index.py",
            ".system/modules/localhost.py",
            ".system/sources/clight.json",
            ".system/sources/hosts",
            ".system/sources/logo.ico",
            ".system/sources/vhosts.conf"
        ],
    },
    include_package_data=True,
    author="Irakli Gzirishvili",
    author_email="gziraklirex@gmail.com",
    description="XAMPHP is a command-line interface app designed to launch PHP projects via XAMPP engine from any location on your Windows machine as a virtual local host",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/IG-onGit/XAMPHP",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
    ],
)
